#  Copyright (c) XiaoLinpeng 2020.

# !/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   parse_utils.py
@Contact :   1553990434@qq.com
@License :   (C)Copyright 2019-Present, XiaoLinpeng

@Modify Time      @Author    @Version    @Description
------------      -------    --------    -----------
2020-04-08 22:09   肖林朋      1.0         不同格式文件解析
"""
import json
from datafushion.args import FileExtractFormatEnum


def parse_file_2_map_list(file_path, file_type):
    if file_type == FileExtractFormatEnum.CSV.value:
        return __parse_sv_file_2_map_list(file_path, ',')
    elif file_type == FileExtractFormatEnum.TSV.value:
        return __parse_sv_file_2_map_list(file_path, '\t')
    elif file_type == FileExtractFormatEnum.JSON.value:
        return __parse_json_2_map_list(file_path)
    elif file_type == FileExtractFormatEnum.GENERAL.value:
        return __parse_general_file_2_map_list(file_path)


def __parse_sv_file_2_map_list(file_path, separator):
    """
    解析Csv文件至MapList形式
    :param file_path:CSV文件路径
    :return: 数据的MapList
    """
    res = []
    index_columns_map = {}
    with open(file_path, 'r', encoding='utf-8') as f:
        read_lines = f.read().splitlines()
        columns = read_lines.pop(0)
        split = columns.split(separator)
        for index, value in enumerate(split):
            index_columns_map[index] = value
        for item in read_lines:
            columns = item.split(separator)
            item_map = {}
            for k, v in index_columns_map.items():
                item_map[v] = columns[k]
            res.append(item_map)

    return res


def __parse_json_2_map_list(file_path):
    """
    解析Json文件至MapList形式
    :param file_path:JSON文件路径
    :return: 数据的MapList
    """
    with open(file_path, 'r', encoding='utf-8') as f:
        content = f.read()
        if content.startswith("[") and content.endswith("]"):
            res = json.loads(content)
        else:
            trans = "[" + content + "]"
            res = json.loads(trans)

    return res


def __parse_general_file_2_map_list(file_path):
    """
        解析普通文件至MapList形式
        :param file_path:JSON文件路径
        :return: 数据的MapList
        """
    res = []
    with open(file_path, 'r', encoding='utf-8') as f:
        content = f.read()
        content_map = {
            'value': content
        }
        res.append(content_map)
    return res



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   args.py
@Contact :   1553990434@qq.com
@License :   (C)Copyright 2019-Present, XiaoLinpeng

@Modify Time      @Author    @Version    @Description
------------      -------    --------    -----------
2020-04-08 21:26   肖林朋      1.0         参数相关实体类
"""

import json
from enum import Enum


#  Copyright (c) XiaoLinpeng 2020.

class ArgStruct(object):
    """
    总参数结构体
    """


    def __init__(self):
        self.param = None
        self.output = None
        self.input = None


    @staticmethod
    def parse(arg):
        """
        解析参数
        :param arg: 参数数组
        :return: 参数结构体对象
        """
        length = len(arg)
        struct = ArgStruct()
        if length == 1:
            struct.input = arg[0]
        elif length == 2:
            struct.input = arg[0]
            struct.output = arg[1]
        elif length == 3:
            struct.input = arg[0]
            struct.output = arg[1]
            struct.param = arg[2]
        return struct


    def get_input(self):
        """
        获取可用的输入,由于Linux环境可能对引号敏感
        :return: 规范化输入
        """
        new_input = self.input.replace("**", "\"")
        if "}}]" not in new_input:
            new_input = new_input.replace("]", "}}]")
        return new_input


    def get_output(self):
        """
        获取可用的输出,由于Linux环境可能对引号敏感
        :return: 规范化输出
        """
        return self.output.replace("**", "\"")


    def get_param(self):
        """
        获取可用的参数,由于Linux环境可能对引号敏感
        :return: 规范化参数
        """
        return self.param.replace("**", "\"")


    def fetch_input_structs(self):
        """
        获取输入结构体
        :return:
        """
        res = []
        if self.input is not None:
            for item in json.loads(self.get_input()):
                input_struct = InputStruct()
                input_struct.file_path = item['filePath']
                input_struct.file_format = item['fileFormat']
                input_struct.flag_mapping = item['flagMapping']
                input_struct.mapping = item['mapping']
                res.append(input_struct)
        return res


    def fetch_param_map(self):
        """
        获取参数Map
        :return:
        """
        res = {}
        if self.param is not None:
            res = ParamParse.parse_params(self.param)
        return res


    def fetch_output_path(self):
        """
        获取输出路径
        :return: 输出路径
        """
        return self.output


class InputStruct(object):
    """
    输入结构体
    """


    def __init__(self):
        self.file_path = None
        self.file_format = None
        self.flag_mapping = None
        self.mapping = None


    def mapping_2_json(self, cancel_connect_id):
        """
        mapping的字符串转换为Json对象
        :return: json对象
        """
        if cancel_connect_id:
            del self.mapping['connectId']
        return self.mapping


    def mapping_2_multi_value(self, cancel_connect_id):
        """
        映射为多值Mapping,key为输入数据的字段,值为算法输入的字段
        :return: 多值map
        """
        res = dict()
        json_mapping = self.mapping_2_json(cancel_connect_id)
        for k, v in json_mapping.items():
            if res.__contains__(v):
                value_list = res[v]
                value_list.append(k)
            else:
                value_list = list()
                value_list.append(k)
                res[v] = value_list
        return res


class ParamEnum(Enum):
    """
    参数枚举
    """
    STRING = 'String'
    INT = 'Int'
    DOUBLE = 'Double'


class FileExtractFormatEnum(Enum):
    """
    文件提取格式枚举
    """
    JSON = "json"
    CSV = "csv"
    TSV = "tsv"
    GENERAL = "general"
    PARQUET = "parquet"
    AVRO = "avro"
    ORC = "orc"
    MODEL = "model"


class ParamParse(object):
    """
    参数结构体
    """


    @staticmethod
    def parse_params(params):
        """
        解析参数
        :param params: 参数字符串
        :return: 参数Map
        """
        param_map = {}
        param_splits = params.split(',')
        for param_body in param_splits:
            key_value_splits = param_body.split("->")
            key = key_value_splits[0]
            value_body = key_value_splits[1]

            value_type_splits = value_body.split(":")
            param_type = value_type_splits[1]

            if ParamEnum.STRING.value == param_type:
                param_map[key] = value_type_splits[0]
            elif ParamEnum.INT.value == param_type:
                param_map[key] = int(value_type_splits[0])
            elif ParamEnum.DOUBLE.value == param_type:
                param_map[key] = float(value_type_splits[0])

        return param_map



#  Copyright (c) XiaoLinpeng 2020.

# !/usr/bin/env python
# -*- encoding: utf-8 -*-
"""
@File    :   handle.py    
@Contact :   1553990434@qq.com
@License :   (C)Copyright 2019-Present, XiaoLinpeng

@Modify Time      @Author    @Version    @Description
------------      -------    --------    -----------
2020-04-09 19:12   肖林朋      1.0         数据处理封装闭包管理器
"""
from datafushion.args import ArgStruct
from datafushion.parse_utils import parse_file_2_map_list

from contextlib import contextmanager
import json
import sys


class HandleInputDataStruct(object):
    def __init__(self, file_type, file_path, file_input_mapping, data_list):
        """
        待处理的输入数据结构体
        :param file_type:  文件类型
        :param file_path:  文件路径
        :param file_input_mapping: 输入映射
        :param data_list:  数据集
        """
        self.file_type = file_type
        self.file_path = file_path
        self.file_input_mapping = file_input_mapping
        self.data_list = data_list


class HandleDataSet(object):
    def __init__(self, input_data_struct_list, output_path, param_map, data_result):
        """
        待处理数据集
        :param input_data_struct_list: 输入数据结构体列表
        :param output_path:  输出路径
        :param param_map:  参数映射
        :param data_result:  数据合并结果集
        """
        self.input_data_struct_list = input_data_struct_list
        self.output_path = output_path
        self.param_map = param_map
        self.data_result = data_result


@contextmanager
def operation(reverse_mapping=False):
    """
    数据处理封装
    :return:
    """


    def destruction():
        """
        处理输入,输入可以有多个数据集
        :return: 输出处理结构体,输入数据集合,输出路径,参数Map
        """
        args = None
        if len(sys.argv) == 3:
            args = [sys.argv[1], sys.argv[2], None]
        elif len(sys.argv) == 4:
            args = [sys.argv[1], sys.argv[2], sys.argv[3]]

        arg_struct = ArgStruct.parse(args)
        input_structs = arg_struct.fetch_input_structs()
        output_path = arg_struct.fetch_output_path()
        param_map = arg_struct.fetch_param_map()

        input_data_struct_list = []
        for input_struct in input_structs:
            file_type = input_struct.file_format
            file_path = input_struct.file_path
            if reverse_mapping:
                file_input_mapping = input_struct.mapping_2_json(True)
            else:
                file_input_mapping = input_struct.mapping_2_multi_value(True)

            data_list = parse_file_2_map_list(file_path, file_type)
            input_data_struct_list.append(HandleInputDataStruct(file_type=file_type, file_path=file_path,
                                                                file_input_mapping=file_input_mapping,
                                                                data_list=data_list))

        return HandleDataSet(input_data_struct_list=input_data_struct_list, output_path=output_path,
                             param_map=param_map, data_result=None)


    def save_data(data_result, output_path):
        """
        存储数据
        :param data_result: 已处理好的数据
        :param output_path:  输出路径
        :return:
        """
        with open(output_path, "w", encoding='utf-8') as f:
            json.dump(data_result, f)


    destruction = destruction()  # type: HandleDataSet
    yield destruction

    save_data(destruction.data_result, destruction.output_path)
